#!/usr/bin/env python

from distutils.core import setup

setup(name='glicko2',
      version='1.0',
      description='Python implementation of glicko2',
      author='Alexander Nordlund',
      author_email='deep.alexander@gmail.com',
      url='https://bitbucket.org/deepy/glicko2//',
      packages=['glicko2',],
     )
